
-- Gadget that makes compulsory to have built N units of certain kinds to unlock some (build)buttons.

-- Change by the name of your research buildings
key_units = {UnitDefNames.tllgeo.id,UnitDefNames.tllgeo1.id,UnitDefNames.tllgeo2.id,UnitDefNames.tllgeo3.id}
key_nbr = 8 -- how many you need to unlock the tech

-- Change by the names of the units that need the research structure. Mind the minus sign.
-- Can also use command IDs here, such as CMD.CLOAK (without a - this time).
locked_units = {-UnitDefNames.apart.id}


function gadget:GetInfo()
	return {
		name = "PA Source locker",
		desc = "A Source Lock with Count",
		author = "zwzsg, Senna",
		date = "October 2009, July 2016",
		license = "Public domain",
		layer = -1,
		enabled = true
	}
end

if (gadgetHandler:IsSyncedCode()) then

	-- Local vars
	local key_count = {}
	local ori_desc = {}
	local str_cond = nil
	local unlocked = {}

	local function LockButtons(u)
		local function DisabledToolTip(u,ucd,udd)
			if not ori_desc[udd] then
				ori_desc[udd]=Spring.GetUnitCmdDescs(u)[ucd].tooltip
			end
			if not str_cond then
				local hn={}
				for _,ku in ipairs(key_units) do
					table.insert(hn,UnitDefs[ku].humanName)
				end
				str_cond="Requires "..key_nbr.." "..table.concat(hn," or ")
			end
			return "\255\255\64\64"..str_cond.."!\n\255\255\255\255"..ori_desc[udd]
		end
		for _,udd in ipairs(locked_units) do
			local UnitCmdDesc = Spring.FindUnitCmdDesc(u,udd)
			if UnitCmdDesc then
				Spring.EditUnitCmdDesc(u,UnitCmdDesc,{disabled=true,tooltip=DisabledToolTip(u,UnitCmdDesc,udd)})
			end
		end
	end

	local function UnlockButtons(u)
		local function EnabledToolTip(u,ucd,udd)
			if not ori_desc[udd] then
				ori_desc[udd]=Spring.GetUnitCmdDescs(u)[ucd].tooltip
			end
			return ori_desc[udd]
		end
		for _,udd in ipairs(locked_units) do
			local UnitCmdDesc = Spring.FindUnitCmdDesc(u,udd)
			if UnitCmdDesc then
				Spring.EditUnitCmdDesc(u,UnitCmdDesc,{disabled=false,tooltip=EnabledToolTip(u,UnitCmdDesc,udd)})
			end
		end
	end

	function gadget:AllowCommand(u,ud,team,cmd,param,opt,synced)
		for _,udd in ipairs(locked_units) do
			if cmd==udd and not unlocked[team] then
				return false
			end
		end
		return true
	end

	function gadget:AllowUnitCreation(ud,builder,team,x,y,z)
		for _,udd in ipairs(locked_units) do
			if -ud==udd and not unlocked[team] then
				return false
			end
		end
		return true
	end

	function gadget:UnitCreated(u,ud,team)
		if not unlocked[team] then
			LockButtons(u)
		end
	end

	function gadget:UnitFinished(u,ud,team)
		for _,udd in ipairs(key_units) do
			if udd==ud then
				key_count[team]=(key_count[team] or 0)+1
				if key_count[team]>=key_nbr and not unlocked[team] then
					for _,v in ipairs(Spring.GetTeamUnits(team)) do
						UnlockButtons(v)
					end
					unlocked[team]=true
					Spring.SendMessageToTeam(team,"Particle Accelerator Unlocked!")
				end
			end
		end
	end

	local function isComplete(u)
		local health,maxHealth,paralyzeDamage,captureProgress,buildProgress=Spring.GetUnitHealth(u)
		if buildProgress and buildProgress>=1 then
			return true
		else
			return false
		end
	end

	function gadget:UnitDestroyed(u,ud,team)
		if isComplete(u) then
			for _,udd in ipairs(key_units) do
				if udd==ud then
					key_count[team]=(key_count[team] or 0)-1
					if key_count[team]<key_nbr and unlocked[team] then
						for _,v in ipairs(Spring.GetTeamUnits(team)) do
							LockButtons(v)
						end
						unlocked[team]=false
						Spring.SendMessageToTeam(team,"Particle Accelerator lost!")
					end
				end
			end
		end
	end

	function gadget:UnitGiven(u,ud,team,oldteam)
		gadget:UnitCreated(u,ud,team)
		if unlocked[team] then
			UnlockButtons(u)
		end
		if isComplete(u) then
			gadget:UnitFinished(u,ud,team)
		end
	end

	function gadget:UnitTaken(u,ud,team,newteam)
		gadget:UnitDestroyed(u,ud,team)
	end

end
